﻿//Скрипт предназначен для открытия в Adobe Photoshop растровых файлов, помещенных в публикацию Adobe InDesign.
//Опционально возможен запуск одного из экшенов для обработки файлов, а также закрытие файлов после обработки.
//Версия 1.7 доработана под CS4. 
//Добавлена функция автоматического приведения разрешения изображений к 300 dpi в фотошопе (для изображений с эфф.разрешением менее 260 dpi или более 350 dpi)
//Добавлена функция Merge Visible Layers
//Добавлена функция cохранения PSD копии оригинала изображения
//АВТОР СКРИПТА: Олег Бутрин (http://indesign.rudtp.ru)
//ДОРАБОТКА СКРИПТА: Леонид Пивнюк (Leo3001@mail.ru)
//ДАТА НАПИСАНИЯ: 05.02.2006
//ДАТА ДОРАБОТКИ: 16.03.2012

#target indesign
 
//Заголовочная часть - определение переменных диалога
var langAlertNoDoc = localize({en: "No documents are open!", ru: "Нет открытых документов!"});
var langAlertPhotoshopNotStarted = localize({en: "Photoshop not started!", ru: "Невозможно запустить Photoshop!"});
var langAlertNoImages = localize({en: "Can\'t find normal link!", ru: "Невозможно найти обновленные связи!"});

var langDialogImagesFrom = localize({en: "Images from:", ru: "Картинки из:"});
var langDialogDocument = localize({en: "document", ru: "документа"});
var langDialogSpread = localize({en: "spread", ru: "разворота"});
var langDialogPage = localize({en: "page", ru: "страницы"});
var langDialogSelection = localize({en: "selection", ru: "выделенного"});
var langDialogResize = localize({en: "Resize", ru: "Менять размер"});
var langDialogMerge = localize({en: "Merge Visible Layers", ru: "Слить видимые слои"});
var langDialogCopy = localize({en: "Save PSD copy", ru: "Сохранять PSD копию"});
var langDialogRunAction = localize({en: "Run Action", ru: "Запустить Action"});
var langDialogCloseAfterAction = localize({en: "Close after Action", ru: "Закрыть после выполнения"});


var iniScriptName = "Open Files In PhotoShop";
var iniScriptVersion = "1.7";

//Для изменения экшена отредактируй нижеследующие определения
var iniActionSet = "Default Actions";
var iniActionName = "Quadrant Colors";

//var myObject = selection
//alert(myObject.width);

//Функция получения картинок из указанного массива объектов
function getSelectedImages(myArray) {
	var myResult = new Array();
	for (var myCounter = 0; myCounter < myArray.length; myCounter++) {
		var myObject = myArray[myCounter];
		if (myObject.constructor.name == "Image") {
			myResult.push(myObject);
		} else {
			for (var objCounter = 0; objCounter < myObject.allGraphics.length; objCounter++) {
				if (myObject.allGraphics[objCounter].constructor.name == "Image") {
					myResult.push(myObject.allGraphics[objCounter]);
				}
			}
		}
	}
	return myResult;
}

//Функция поиска дублирующихся файлов в массиве 
function checkDoubledFile(myArray, myObject) {
	for (var objCounter = 0; objCounter < myArray.length; objCounter++) {
		if (myObject.itemLink.filePath == myArray[objCounter].itemLink.filePath) {
			return true;
		}
	}
	return false;
}

//Функция получения списка неповторяющихся файлов 
function getUniqImages(myArray) {
	var myResult = new Array();
	for (var myCounter = 0; myCounter < myArray.length; myCounter++) {
		var myObject = myArray[myCounter];
		if (!checkDoubledFile(myResult, myObject)) {
			myResult.push(myObject);
		}
	}
	return myResult;
}
//Функция отсеивания необновленных и отсутствующих файлов
function removeMissingFile(myArray) {
	var myResult = new Array();
	for (var myCounter = 0; myCounter < myArray.length; myCounter++) {
		if (myArray[myCounter].itemLink.status == LinkStatus.normal)  {
			myResult.push(myArray[myCounter]);
		}
	}
	return myResult;
}
//Функция получения полного пути к файлу
function getImagePath(myArray) {
	var myResult = new Array();
	var myFile
	var myOpenFile
	for (var myCounter = 0; myCounter < myArray.length; myCounter++) {
		myFile = File(myArray[myCounter].itemLink.filePath);
		myOpenFile = myFile.path + "/" +myFile.name;
		myResult.push(myOpenFile);
	}
	return myResult;
}
//Функция запуска приложения с использованием функций BrigeTalk (подробно об использовании Bridge Talk смотри в описании Bridge Scripting)
function executeTarget (appSpec) {
	try {
//		если приложение уже запущено, передаем ему фокус
		if (BridgeTalk.isRunning(appSpec)) {
			BridgeTalk.bringToFront (appSpec);
		} else {
//			если приложение не запущено, запускаем его и передаем фокус
			BridgeTalk.launch (appSpec);
			BridgeTalk.bringToFront (appSpec);
		}
		return true;
	} catch (error) {
		return false;
	}
}

with (app) {
//Получаем активный документ
	try {
		var myDoc = activeDocument;
	} catch (error) {
		alert(langAlertNoDoc);
		exit();
	}
//	Делаем выборку файлов из выделенных объектов и проводим отсеивание
	var myFiles = getSelectedImages(selection);
	myFiles = getUniqImages(myFiles);
	myFiles = removeMissingFile(myFiles);
//	Создаем и запускаем диалог
/*	var myDialog = dialogs.add({name: iniScriptName + " " + iniScriptVersion});
	with (myDialog.dialogColumns.add()) 
		{
		with (borderPanels.add().dialogColumns.add()) 
			{
			dialogRows.add().staticTexts.add({staticLabel:langDialogImagesFrom});
			var selectedRegion = dialogRows.add().radiobuttonGroups.add();
			with (selectedRegion) 
				{
				radiobuttonControls.add({staticLabel:langDialogDocument});
				radiobuttonControls.add({staticLabel:langDialogSpread});
				radiobuttonControls.add({staticLabel:langDialogPage, checkedState: true});
				if (myFiles.length > 0) 
					{
					radiobuttonControls.add({staticLabel:langDialogSelection, checkedState: true});
					}
				}
			}
		with (borderPanels.add().dialogColumns.add()) 
			{
			var selectedResize = dialogRows.add().checkboxControls.add({staticLabel: langDialogResize, checkedState: true});
			var selectedMerge = dialogRows.add().checkboxControls.add({staticLabel: langDialogMerge, checkedState: true});
			var selectedCopy = dialogRows.add().checkboxControls.add({staticLabel: langDialogCopy, checkedState: true});
			var selectedAction = dialogRows.add().checkboxControls.add({staticLabel: langDialogRunAction});
			var selectedCloseAfter = dialogRows.add().checkboxControls.add({staticLabel: langDialogCloseAfterAction});
			}
		dialogRows.add().staticTexts.add({staticLabel:"\u00A9 Олег Бутрин"});
		dialogRows.add().staticTexts.add({staticLabel:"\u00A9 Леонид Пивнюк"});
		dialogRows.add().staticTexts.add({staticLabel:"Leo3001@mail.ru"});
		}
	
	var myResult = myDialog.show();
	if (!myResult) {
		myDialog.destroy();
		exit();
	}

//	Проводим выборку файлов и отсеивание на основании выбранных настроек
	switch (selectedRegion.selectedButton) {
		case 0:
			var myFiles = getSelectedImages([myDoc]);
			myFiles = getUniqImages(myFiles);
			myFiles = removeMissingFile(myFiles);
			break;
		case 1:
			var myFiles = getSelectedImages([activeWindow.activeSpread]);
			myFiles = getUniqImages(myFiles);
			myFiles = removeMissingFile(myFiles);
			break;
		case 2:
			var myFiles = getSelectedImages([activeWindow.activePage]);
			myFiles = getUniqImages(myFiles);
			myFiles = removeMissingFile(myFiles);
			break;
	}
*/
//	Если в выборке не осталось ни одного файла, то предупреждаем и выходим
	if (myFiles.length == 0) {
		alert(langAlertNoImages);
		exit();
	}

var myHorSizes = new Array();
var myVertSizes = new Array();
var myEffPpis = new Array();
for (var myCounter = 0; myCounter < myFiles.length; myCounter++) {
	if (myFiles[myCounter].rotationAngle == 0)
		{
		var HorSize = myFiles[myCounter].geometricBounds[3] - myFiles[myCounter].geometricBounds[1];
		var VertSize = myFiles[myCounter].geometricBounds[2] - myFiles[myCounter].geometricBounds[0];
		var HorPpi = myFiles[myCounter].effectivePpi[0]
		var VertPpi = myFiles[myCounter].effectivePpi[1]
		}
	else
		{
		myFiles[myCounter].rotationAngle = 0;
		var HorSize = myFiles[myCounter].geometricBounds[3] - myFiles[myCounter].geometricBounds[1];
		var VertSize = myFiles[myCounter].geometricBounds[2] - myFiles[myCounter].geometricBounds[0];
		var HorPpi = myFiles[myCounter].effectivePpi[0]
		var VertPpi = myFiles[myCounter].effectivePpi[1]
		myDoc.undo()
		};
/*	if (HorPpi < 230 || VertPpi < 230) {
		try {myFiles[myCounter].itemLink.show();}
		catch (e) {app.selection[0] = myFiles[myCounter];}
		alert ("Фото низкого разрешения:  " + HorPpi + " dpi");
		} 
	if (HorPpi < 260 || HorPpi > 350 || VertPpi < 260 || VertPpi > 350) {
			myEffPpis.push(true)
		} else { myEffPpis.push(false)
			}*/	
	myHorSizes.push(HorSize);
	myVertSizes.push(VertSize);
	}

//var HorSize = myFiles[0].geometricBounds[0] - myFiles[0].geometricBounds[2];
//var VertSize = myFiles[0].geometricBounds[3] - myFiles[0].geometricBounds[1];
//var temptxt  = temptxt1 + "   " + temptxt2;
//alert(myHorSizes[1] + "   " + myVertSizes[1]);

//var myHor = myFiles[0].effectivePpi[0] * myFiles[0].absoluteHorizontalScale
//alert(myHor + ",  " + myFiles[1].absoluteHorizontalScale);

//	Создаем в виде текстовой переменной скрипт для Photoshop очень аккуратно и внимательно, посколько любая ошибка приводит к отказу выполнения 
	var myScriptString = "";
//	Получаем список имен файлов и передаем его в запускаемый скрипт
	myScriptString += "var myFiles = " + getImagePath(myFiles).toSource() + ";";
	myScriptString += "var myHorSizes = " + myHorSizes.toSource() + ";";
	myScriptString += "var myVertSizes = " + myVertSizes.toSource() + ";";
	myScriptString += "var myEffPpis = " + myEffPpis.toSource() + ";";
	myScriptString += "for (var myCounter = 0; myCounter < myFiles.length; myCounter++) { ";
	myScriptString += "try {";
	myScriptString += "app.open(File(myFiles[myCounter]));";
//	myScriptString += "alert(myFiles);";
//      Если нужно сохранить сопию оригинала
/*	if (selectedCopy.checkedState) {
		myScriptString += "psSaveOptions = new PhotoshopSaveOptions();";
		myScriptString += "{psSaveOptions.alphaChannels = false;";
		myScriptString += "psSaveOptions.annotations = false;";
		myScriptString += "psSaveOptions.embedColorProfile = true;";
		myScriptString += "psSaveOptions.layers = true;";
		myScriptString += "psSaveOptions.maximizeCompatibility = true;";
		myScriptString += "psSaveOptions.spotColors = true; }";
		myScriptString += "myFile = new File (app.activeDocument.fullName + '_copy.psd');";
		myScriptString += "app.activeDocument.saveAs(myFile, psSaveOptions, true, Extension.LOWERCASE);";
	}
//	Передаем экшн
	if (selectedAction.checkedState) {
		myScriptString += "app.doAction(\'" + iniActionName + "\', \'" + iniActionSet + "\');";
	}
//      Если нужно слить слои
	if (selectedMerge.checkedState) {
		myScriptString += "try {";
		myScriptString += "app.activeDocument.mergeVisibleLayers ();";
		myScriptString += "} catch (error) {}";
	}
//      Если нужно поменять разрешение на 300 dpi
	if (selectedResize.checkedState) {
		myScriptString += "preferences.rulerUnits = Units.MM;";
		myScriptString += "if (myEffPpis[myCounter]) { ";
		myScriptString += "app.activeDocument.resizeImage (myHorSizes[myCounter], myVertSizes[myCounter], 300) };";
	}
//      Если нужно закрыть после обработки
	if (selectedCloseAfter.checkedState) {
		myScriptString += "app.activeDocument.close(SaveOptions.SAVECHANGES);";
	}*/
	myScriptString += "} catch (error) {alert(error)}";
	myScriptString += "}";
//	Создаем и выполняем объект BridgeTalk	
	if (executeTarget("photoshop")) {
		var bt = new BridgeTalk;
		bt.target = "photoshop";
		bt.body = myScriptString;
		bt.send();
	} else {
		alert(langAlertPhotoshopNotStarted);
//		myDialog.destroy();
		exit();
	}	
//	myDialog.destroy();
	exit();
}